/*
 * Copyright 2010 Sergey Martynov http://studiomobile.ru
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.studiomobile.http;

import org.apache.http.ConnectionClosedException;
import org.apache.http.HttpException;
import org.apache.http.nio.ContentDecoder;
import org.apache.http.nio.ContentEncoder;
import org.apache.http.nio.NHttpClientConnection;
import org.apache.http.nio.NHttpClientHandler;

import java.io.IOException;

public class HttpClientConnectionHandler implements NHttpClientHandler {

    private final NHttpClientHandler handler;

    public HttpClientConnectionHandler(NHttpClientHandler handler) {
        this.handler = handler;
    }

    public void connected(NHttpClientConnection connection, Object attachment) {
        this.handler.connected(connection, attachment);
        HttpRequestFutureImpl state = (HttpRequestFutureImpl) attachment;
        state.setActiveConnection(connection);
    }

    public void closed(NHttpClientConnection conn) {
        HttpRequestFutureImpl state = getState(conn);
        if (!state.isDone() && conn == state.getActiveConnection()) {
            state.failed(new ConnectionClosedException("Connection closed"));
        }
        this.handler.closed(conn);
    }

    public void requestReady(NHttpClientConnection conn) {
        this.handler.requestReady(conn);
    }

    public void outputReady(NHttpClientConnection conn, ContentEncoder encoder) {
        this.handler.outputReady(conn, encoder);
    }

    public void responseReceived(NHttpClientConnection conn) {
        this.handler.responseReceived(conn);
    }

    public void inputReady(NHttpClientConnection conn, ContentDecoder decoder) {
        this.handler.inputReady(conn, decoder);
    }

    public void exception(NHttpClientConnection conn, HttpException ex) {
        getState(conn).failed(ex);
        this.handler.exception(conn, ex);
    }

    public void exception(NHttpClientConnection conn, IOException ex) {
        getState(conn).failed(ex);
        this.handler.exception(conn, ex);
    }

    public void timeout(NHttpClientConnection conn) {
        getState(conn).failed(new ConnectionClosedException("Connection timed out"));
        this.handler.timeout(conn);
    }

    private HttpRequestFutureImpl getState(NHttpClientConnection conn) {
        return HttpRequestExecutionHandler.getState(conn.getContext());
    }
}
