/*
 * Copyright 2010 Sergey Martynov http://studiomobile.ru
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.studiomobile.http;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpRequest;
import org.apache.http.message.BasicHttpEntityEnclosingRequest;
import org.apache.http.message.BasicHttpRequest;

import java.io.UnsupportedEncodingException;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;

public class HttpClientHelper {

    public static HttpRequest buildRequest(String method, URL url, Header... headers) {
        BasicHttpRequest request = new BasicHttpRequest(method, url.toExternalForm());
        for (Header header : headers) {
            request.addHeader(header);
        }
        return request;
    }

    public static HttpEntityEnclosingRequest buildRequest(String method, URL url, HttpEntity entity, Header... headers) {
        BasicHttpEntityEnclosingRequest request = new BasicHttpEntityEnclosingRequest(method, url.toExternalForm());
        request.setEntity(entity);
        for (Header header : headers) {
            request.addHeader(header);
        }
        return request;
    }

    public static InetSocketAddress getAddress(URL url) {
        return new InetSocketAddress(url.getHost(), getPort(url));
    }

    protected static int getPort(URL url) {
        return url.getPort() == -1 ? url.getDefaultPort() : url.getPort();
    }

    public static URL buildURL(URL url, HttpParam... params) throws MalformedURLException {
        if (params.length == 0) return url;
        StringBuilder builder = new StringBuilder();
        if (url.getPath() != null) {
            builder.append(url.getPath());
        }
        boolean firstParam = true;
        if (url.getQuery() != null) {
            builder.append("?").append(url.getQuery());
            firstParam = false;
        }
        for (HttpParam param : params) {
            if (param.getValue() != null) {
                builder.append(firstParam ? "?" : "&").append(encode(param.getName()));
                builder.append("=").append(encode(param.getValue()));
                firstParam = false;
            }
        }
        if (url.getRef() != null) {
            builder.append("#").append(url.getRef());
        }
        if (url.getPort() == -1) {
            return new URL(url.getProtocol(), url.getHost(), url.getPort(), builder.toString());
        } else {
            return new URL(url.getProtocol(), url.getHost(), builder.toString());
        }
    }

    public static String encode(String s) {
        try {
            return URLEncoder.encode(s, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            return s;
        }
    }
}
