/*
 * Copyright 2010 Sergey Martynov http://studiomobile.ru
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.studiomobile.http;

import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.StatusLine;
import org.apache.http.nio.reactor.IOSession;
import org.apache.http.protocol.HttpContext;

import java.io.IOException;

class HttpRequestExecutionHandler implements org.apache.http.nio.protocol.HttpRequestExecutionHandler {

    public interface RedirectHandler {
        void handleRedirect(HttpRequestFutureImpl state, HttpContext context) throws IOException;
    }

    private final RedirectHandler redirectHandler;

    public HttpRequestExecutionHandler(RedirectHandler redirectHandler) {
        this.redirectHandler = redirectHandler;
    }

    public void initalizeContext(HttpContext context, Object attachment) {
    }

    public void finalizeContext(HttpContext context) {
    }

    public static HttpRequestFutureImpl getState(HttpContext context) {
        return (HttpRequestFutureImpl) context.getAttribute(IOSession.ATTACHMENT_KEY);
    }

    public HttpRequest submitRequest(HttpContext context) {
        HttpRequestFutureImpl state = getState(context);
        return state.isCancelled() ? null : state.getRequest();
    }

    public void handleResponse(HttpResponse response, HttpContext context) throws IOException {
        HttpRequestFutureImpl state = getState(context);
        if (state.isDone()) return;
        StatusLine status = response.getStatusLine();
        if (isRedirect(status) && state.redirect(response)) {
            redirectHandler.handleRedirect(state, context);
        } else {
            state.finished(response);
        }
    }

    private static boolean isRedirect(StatusLine status) {
        switch (status.getStatusCode()) {
            case HttpStatus.SC_MOVED_PERMANENTLY:
            case HttpStatus.SC_MOVED_TEMPORARILY:
            case HttpStatus.SC_SEE_OTHER:
            case HttpStatus.SC_TEMPORARY_REDIRECT:
                return true;
        }
        return false;
    }
}
